#!/usr/bin/perl
# 'use lib' order matters! inserted at front of include path each time - sort
# in 'increasing importance' order (environment should overrule all other, etc)
use lib "$ENV{PWD}/";
use lib "/opt/nordugrid/libexec";
# 'Make' (*not* autoconf!) expands this one to avoid hardcoding. Please refer to:
# http://www.gnu.org/software/autoconf/manual/autoconf-2.57/html_chapter/autoconf_4.html#SEC22
use lib "/opt/nordugrid/libexec";
use lib "$ENV{NORDUGRID_LOCATION}/libexec";

use infosys_shared;
# import resource gateway configuration
use gw_shared qw(%gw_config);
use InfosysCluster qw(%config &clusterldif);
#use warnings;

# populates the nordugrid-cluster LDAP entry with real GM, config 
# and fork LRMS info
# The GM interface is factored into the InfosysCluster module.
# Don't use 'my xyz' for the interfacing GLOBAL variables!

# Make sure gateway is correctly configured
if(! $gw_config{gateway_enabled}) {
	$err_string="lrms configured to use remote-pbs but remote tools not enabled!";
	&infosys_shared::write_log($err_string);
	die "$err_string\n";
}

$lrms_type = "fork";

# lrms_version has no meaning for fork therefore it is not set
#$lrms_version ="1.0";


# number of cpus is calculated from the /proc/cpuinfo, by default it is set to 1
$totalcpus = 0;
#unless (open CPUINFOFILE, "</proc/cpuinfo") {
unless (open CPUINFOFILE, "$gw_config{remote_exec} -c $gw_config{gateway_conf} 'cat /proc/cpuinfo'  |") {
   $config{loglevel} and  &infosys_shared::write_log("can't read the /proc/cpuinfo");
   die "can't read the /proc/cpuinfo";
}   
while(my $line = <CPUINFOFILE>) {
    chomp($line);
    if ($line =~ m/^processor/) {
        $totalcpus++;	
    }
}
close CPUINFOFILE;
$totalcpus ||= 1;

# it is assumed that all the cpus are sitting in the same machine
$cpudistribution = $totalcpus."cpu:1";

# usedcpus on a fork machine is determined from the 1min cpu loadaverage and can not be
# larger than the totalcpus
#unless (open LOADAVGFILE, "</proc/loadavg") {
unless (open LOADAVGFILE, "$gw_config{remote_exec} -c $gw_config{gateway_conf} 'cat /proc/loadavg'  |") {
   $config{loglevel} and  &infosys_shared::write_log("can't read the /proc/loadavg");
   die "can't read the /proc/loadavg";
}
while(my $line = <LOADAVGFILE>) {
    chomp($line);
    ($oneminavg, $fiveminavg, $fifteenminavg, $processnumber, $lastprocessid ) =  split(/\s+/, $line);
}
close LOADAVGFILE;

if (defined $oneminavg) {
   $usedcpus = int $oneminavg;
   if ($usedcpus >= $totalcpus) {
      $usedcpus = $totalcpus;
   }    
}
else {
   $usedcpus  = 0;
}    

# no LRMS queuing jobs on a fork machine, fork has no queueing ability
$lrms_queued = 0;


# $lrms_total (total number of active jobs in a fork system) is calculated by 
# making use of the 'ps axr'
$lrms_total = 0;
#unless (open PSCOMMAND,  "ps axr |") {
unless (open PSCOMMAND,  "$gw_config{remote_exec} -c $gw_config{gateway_conf} 'ps axr' |") {
   $config{loglevel} and  &infosys_shared::write_log("error in executing ps axr");
   die "error in executing ps axr";
}
while(my $line = <PSCOMMAND>) {
    chomp($line);
    next if ($line =~ m/PID TTY/);
    next if ($line =~ m/ps axr/);
    next if ($line =~ m/cluster-fork/);     
    $lrms_total++;
}
close PSCOMMAND;


# for consistency and to overcome the gm latency the $usedcpus
# and the $lrms_total can not be less than the $gmjobs{inlrms}
($usedcpus <= $gmjobs{inlrms}) and $usedcpus = $gmjobs{inlrms};
($lrms_total <= $gmjobs{inlrms}) and $lrms_total = $gmjobs{inlrms};


# generating the cluster ldif  
&clusterldif;


# some performance test
my $runtime =  time - $^T;
if ($config{loglevel} == 2) {
   &infosys_shared::write_log("execution time: $runtime");
}
elsif ($config{loglevel} == 1 and $runtime >= 4 ) {  
   &infosys_shared::write_log("SLOW script: $runtime");  
}
   

exit;
