#!/usr/bin/perl
# 'use lib' order matters! inserted at front of include path each time - sort
# in 'increasing importance' order (environment should overrule all other, etc)
use lib "$ENV{PWD}/";
use lib "/opt/nordugrid/libexec";
# 'Make' (*not* autoconf!) expands this one to avoid hardcoding. Please refer to:
# http://www.gnu.org/software/autoconf/manual/autoconf-2.57/html_chapter/autoconf_4.html#SEC22
use lib "/opt/nordugrid/libexec";
use lib "$ENV{NORDUGRID_LOCATION}/libexec";

use remote_infosys_shared;
use infosys_shared;
# import resource gateway configuration
use gw_shared qw(%gw_config);
use InfosysCluster qw(%config &clusterldif);
#use warnings;


# populates the nordugrid-cluster LDAP entry with config, GM
# and PBS data

#external PBS commands
$pbsnodes_command="$config{pbs_bin_path}/pbsnodes";
$qstat_command="$config{pbs_bin_path}/qstat";
$qmgr_command="$config{pbs_bin_path}/qmgr";

# Make sure gateway is correctly configured
if(! $gw_config{gateway_enabled}) {
	$err_string="lrms configured to use remote-pbs but remote tools not enabled!";
	&infosys_shared::write_log($err_string);
	die "$err_string\n";
}

#determine the flavour and version of PBS"
my $qmgr_string="";

#print("remote qmgr query: $gw_config{remote_exec} -c $gw_config{gateway_conf} '$qmgr_command -c \"list server\"'\n");
$qmgr_string=`$gw_config{remote_exec} -c $gw_config{gateway_conf} '$qmgr_command -c "list server"'`;
if ($? != 0 and $loglevel) {    
    &infosys_shared::write_log("Can't query qmgr");
}
$qmgr_string =~ /pbs_version = \b(\D+)_(\d\S+)\b/;
# TODO: is the 'remote' part desired?
$lrms_type="remote $1";
$lrms_version=$2;


#read the qstat information about all the jobs
#print("remote qstat query: $gw_config{remote_exec} -c $gw_config{gateway_conf} '$qstat_command'\n");
unless (open QSTATOUTPUT,  "$gw_config{remote_exec} -c $gw_config{gateway_conf} '$qstat_command'  |") {
    $loglevel and  &infosys_shared::write_log("Error in executing remote qstat");
    die "Error in executing remote qstat\n";
}
#&infosys_shared::write_log("qstat returned $?");
    
while (my $line= <QSTATOUTPUT>) {       
#      &infosys_shared::write_log("qstat line: $line");
      if ($line =~ m/^\d+/) {
         push @all_pbs_jobs, $line;       	 
      	 }
      }	    
close QSTATOUTPUT;


#processing the pbsnodes output by using a hash of hashes %hoh_pbsnodes
my %hoh_pbsnodes;
&remote_infosys_shared::remote_read_pbsnodes($pbsnodes_command, \%hoh_pbsnodes);


# calculate the totalcpus & cpudistribution & number of used cpus
# taking into account the minor output difference of OpenPBS and PBSPRO
# skipping the nondedicated nodes in case the $dedicated_node_string is defined
$totalcpus=0;
$usedcpus=0;
$cpudistribution="";
@cpudist=0;
if (lc($lrms_type) eq "openpbs"){
     $npstring="np" 
}
elsif (lc($lrms_type) eq "spbs"){
     $npstring="np"
}
elsif (lc($lrms_type) eq "torque"){
     $npstring="np"
}
elsif (lc($lrms_type) eq "pbspro"){
     $npstring="pcpus"
}
else {
     $loglevel and  &infosys_shared::write_log("the given flavour of PBS is not supported");
     die "The given flavour of PBS is not supported"
};  
 
foreach my $node (keys %hoh_pbsnodes){
      my $tmpstring;
      if ($tmpstring = $config{dedicated_node_string}){
         next unless ($hoh_pbsnodes{$node}{"properties"} =~ m/$tmpstring/)	        	  
      }
      $nodestate=$hoh_pbsnodes{$node}{"state"};      
      if ($nodestate=~/down/ or $nodestate=~/offline/ or $nodestate=~/(?:,|^)busy/) {next};      
      $totalcpus += $hoh_pbsnodes{$node}{$npstring};
      $cpudist[$hoh_pbsnodes{$node}{$npstring}] +=1;
      if ($hoh_pbsnodes{$node}{"jobs"}){
        $usedcpus++;
        @comma = ($hoh_pbsnodes{$node}{"jobs"}=~ /,/g);
	$usedcpus+=@comma;
      } 
}      

for (my $i=0; $i<=$#cpudist; $i++) {
   next unless ($cpudist[$i]);  
   $cpudistribution .= " ".$i."cpu:".$cpudist[$i]
}


# Calculate $lrms_queued, the number of jobs (both grid and nongrid) 
# being queued in the LRMS 
@queued_pbs_jobs = grep / [Q,H] /, @all_pbs_jobs;
$lrms_queued = scalar @queued_pbs_jobs;


# Calculate the total number of jobs in the PBS 
# (running, queued, hold, being in any PBS state;
# submitted by both grid and nongrid users): $lrms_total
$lrms_total = scalar @all_pbs_jobs;


# generating the cluster ldif  
&clusterldif;


# some performance test
my $runtime =  time - $^T;
if ($config{loglevel} == 2) {
   &infosys_shared::write_log("execution time: $runtime");
}
elsif ($config{loglevel} == 1 and $runtime >= 4 ) {  
   &infosys_shared::write_log("SLOW script: $runtime");  
}
   


exit;
