#!/usr/bin/perl 
use lib "$ENV{PWD}/";
use lib "$ENV{NORDUGRID_LOCATION}/libexec";
use lib $ENV{NORDUGRID_LOCATION}."/libexec/nordugrid";
use lib "/opt/nordugrid/libexec";
use infosys_shared;
use InfosysCluster qw(%config &clusterldif);
#use warnings;

# populates the nordugrid-cluster LDAP entry with real GM and config but
# dummy LRMS values (13, 66 or "dummy"). 
# The GM interface is factored into the InfosysCluster module.
# Use this template to port the infosys.
# Don't use 'my xyz' for the interfacing GLOBAL variables!


#external SGE commands
$sgenodes_command="$config{sge_bin_path}/qstat -f";
$qstat_command="$config{sge_bin_path}/qstat";
$sgeversion_command="$config{sge_bin_path}/qstat -help";

#<LRMSPORT
# automatically determine the LRMS name(flavour) and version

unless (open SGEVERSIONOUTPUT,  "$sgeversion_command 2>/dev/null |") {
    $config{loglevel} and  &infosys_shared::write_log("Error in determining SGE version");  
    die "Error in determining SGE version \n";				      
}

while (my $line= <SGEVERSIONOUTPUT>) {
     if ($line =~ /^(SGEE{0,2}) (.+)$/i) {
         $lrms_type = $1;
         $lrms_version = $2;
         last;
     }	  
}

close SGEVERSIONOUTPUT;
#LRMSPORT>


#<LRMSPORT
# Calculate the totalcpus & cpudistribution & number of used cpus
# by collecting information from the LRMS
# Consult the schema for the syntax of the $cpudistribution
# If $dedicated_node_string is set, nondedicated nodes, which are not marked with 
# $dedicated_node_string should be skipped


#processing the sgenodes output by using a hash of hashes %hoh_sgenodes
my %hoh_sgenodes;
&infosys_shared::read_sgenodes($sgenodes_command, \%hoh_sgenodes);

$totalcpus=0;
$usedcpus=0;
@cpudist=0;
 
foreach my $node (keys %hoh_sgenodes){

      ($jobi, $load, $arch, $state ) =  split(/\s+/, $hoh_sgenodes{$node});
      ($jobrun, $jobmax) = split(/\//, $jobi);
      if ($state=~/[dDsS]/) {next};      
      $totalcpus += $jobmax;     
      $cpudist[$jobmax] +=1;
      $usedcpus += $jobrun;
}

$run_sge_jobs = $usedcpus;

for (my $i=0; $i<=$#cpudist; $i++) {
   next unless ($cpudist[$i]);  
   $cpudistribution .= " ".$i."cpu:".$cpudist[$i];
}
#LRMSPORT>


#<LRMSPORT
# Calculate $lrms_queued, the number of jobs (both grid and nongrid) 
# being queued in the LRMS 


#read the qstat information about the queued jobs
unless (open QSTATOUTPUT,  "$qstat_command -s p 2>/dev/null  |") {
    $loglevel and  &infosys_shared::write_log("Error in executing qstat");
    die "Error in executing qstat\n";
}
    
my @queued_sge_jobs;
while (my $line= <QSTATOUTPUT>) {       
      if ($line =~ m/^\s\d+/) {
         push @queued_sge_jobs, $line;		     	 
      	 }
      }
	    
close QSTATOUTPUT;
$lrms_queued = @queued_sge_jobs;
#LRMSPORT>


#<LRMSPORT
# Calculate the total number of jobs in the LRMS (running, queued, hold, being in any LRMS state;
# submitted by both grid and nongrid users): $lrms_total
# dummy value for testing:
$lrms_total = $run_sge_jobs + @queued_sge_jobs;
#LRMSPORT>


# generating the cluster ldif  
&clusterldif;


# some performance test
my $runtime =  time - $^T;
if ($config{loglevel} == 2) {
   &infosys_shared::write_log("execution time: $runtime");
}
elsif ($config{loglevel} == 1 and $runtime >= 4 ) {  
   &infosys_shared::write_log("SLOW script: $runtime");  
}
   

exit;
