#!/usr/bin/perl
use strict;
use warnings;

package parse_conf;

# Configuration parsing heavily inspired by InfosysCluster.pl parsing.
# Thus the same comments about a more strict parsing applies.
# Perhaps a more general parsing module can be used in the future.

sub get_conf_value() {
	my %config;
	my $conf_option = $_[0];
	if ($ENV{NORDUGRID_CONFIG}) {
		$config{conf_file} = $ENV{NORDUGRID_CONFIG};
	} else {
		# This fallback path should be correctly set by 'Make' during build
		$config{conf_file} = '/etc/nordugrid.conf';
	}

	unless (open (CONFIGFILE, "<$config{conf_file}")) {
	    print("can't open $config{conf_file} configuration file");
	    die "Can't open $config{conf_file} configuration file\n";
	}

	while (my $line =<CONFIGFILE>) {
	   next if $line =~/^\s*#/;
	   next if $line =~/^\s*\[/;
	   next if $line =~ /^\s*$/;
	   $line=~m/^\s*($conf_option)\s*=\s*"?([^\"\n\#]*)"?/;  
	   
	   next unless $1;
	   
	   #print($line);

	   # In case of multiple appearances of parameter the last one is used
	   $config{$1} = $2;
	}	
	close CONFIGFILE;

	return $config{$conf_option};
}

1;
