#!/usr/bin/perl
#use lib "$ENV{PWD}/";
use lib "$ENV{NORDUGRID_LOCATION}/libexec";
use lib $ENV{NORDUGRID_LOCATION}."/libexec/nordugrid";
use lib "/opt/nordugrid/libexec";
use infosys_shared;
use InfosysQJU qw(%config %hoh_gmjobs %users &queueldif &jobsldif &usersldif);
#use warnings;


# generates everything under the nordugrid-queue-name=xz,
# creates the queue, job & user entries.
# Real GM and config values are used, but dummy values
# are set for LRMS-dependent  attributes (13, 66 or "dummy").
# The GM interface is factored into the InfosysQJU module.
# Use this template to port the infosys.
# Don't use 'my xyz' for the interfacing GLOBAL variables!

#external SGE commands
$sgenodes_command="$config{sge_bin_path}/qstat -f";
$qstat_command="$config{sge_bin_path}/qstat";
$sgeversion_command="$config{sge_bin_path}/qstat -help";
$sgejobinf_command="$config{sge_bin_path}/qstat -F -j ";


$timestamp = time;


#processing the sgenodes output by using a hash of hashes %hoh_sgenodes
my %hoh_sgenodes;
&infosys_shared::read_sgenodes($sgenodes_command, \%hoh_sgenodes);


#<LRMSPORT
# scan the LRMS in order to find out the $totalcpus and the $usedcpus
# Take into account the $dedicated_node_string or the $queue_node_string
# for distinguishing grid and non-grid enabled nodes (skip nonededicated nodes).
# If the queue-level $totalcpus is calculated then the $queue_node_string MUST be set!
# The $totalcpus and the $usedcpus may be needed for the calculation of the per user freecpus.


# calculate the totalcpus & number of used cpus
$totalcpus=0;
my $usedcpus=0;

foreach my $node (keys %hoh_sgenodes){

      ($jobi, $load, $arch, $state ) =  split(/\s+/, $hoh_sgenodes{$node});
      ($jobrun, $jobmax) = split(/\//, $jobi);
      if ($state=~/[dDsS]/) {next};      
      $totalcpus += $jobmax;
      $usedcpus += $jobrun;
}      
#LRMSPORT>


#<LRMSPORT   
# read the queue info such as limits, authorized users' list, etc from the LRMS "queue" command
# into %queue_info and  @acl_users (list of LRMS-authorized unix users, undefined list means
# every unix user is LRMS authorized)
# what can be said about SGE 'queue' limits, how can these be translated?

# queue limits, setting dummy values
# nordugrid-queue-maxrunning
#$queue_info{"max_running"} = 13;
# nordugrid-queue-maxinqueue
# dummy value
#$queue_info{"max_queuable"} = 133;
# nordugrid-queue-maxuserrun
# dummy value
#$queue_info{"max_user_run"} = 13;
# nordugrid-queue-maxcputime (in minutes)
# dummy value
#$queue_info{"max.cput"} = 133;
# nordugrid-queue-mincput (in minutes)
# dummy value
#$queue_info{"min.cput"} = 0;
# nordugrid-queue-defaultcputime (in minutes)
# dummy value
#$queue_info{"default.cput"} = 133;

#LRMSPORT>



#<LRMSPORT
# read the job info from LRMS to a hash of hashes %hoh_lrmsjobs
# The LRMS jobid is the key of the hash, it MUST match the one stored
# in the GM's job.xx.local file as 'localid'
# determine the queue rank of a LRMS queuing job
# $hoh_lrmsjobs{$jobid}{"rank"} = 1 

unless (open SGENODESOUT,  "$sgenodes_command |") {
    &infosys_shared::write_log("error in executing $sgenodes_command");
    die "Error in executing $sgenodes_command";
}

$newnode="";
$newjob="";
$pendingjobs="";
my $queue_rank = 0;
$totalrunning_in_the_queue = 0;
my $runningjobs_in_the_queue = 0;
while (my $line= <SGENODESOUT>) {
    if ($line =~ /^$/) {next};
    if ($line =~ /^\#+$/) {next};
    if ($line =~ /^queuename\s+qtype/) {next};
    if ($line =~ /^-+$/) {
        $newnode = "true";
        $newjob = "";
        $pendingjobs = "";
        next;
    }
    if ($line =~ / PENDING JOBS /) {
        $newnode = "";
        $newjob = "";
        $pendingjobs="true";
        next;
    }
 
    if ($newnode) {
        ($nodeid, $nodetype, $rest ) = split(/\s+/, $line);
        $newnode = "";
        $newjob = "true";
        $pendingjobs = "";
        next;
    }
 
    if ( $newjob ) {
        if ($line =~ /^\s+\d+\s+.*MASTER/) {
            $job_ncpus = 1; #GD just single processes
#           154154     0 l6a180s2b1 b3101dz      r     06/24/2004 09:42:18 MASTER
            ($dum0, $jobid, $dum1, $jobname, $user, $status, $sdate, $stime, $rest ) = split(/\s+/, $line);
 
#           print "$jobid $dum1 $jobname $user $status $rest";
            $users_jobs_running{$user} ++;
            $totalrunning_in_the_queue += $job_ncpus;
            $runningjobs_in_the_queue++;
 
            $hoh_lrmsjobs{$jobid}{"job_state"} = "R";
            $hoh_lrmsjobs{$jobid}{"job_start"} = "$sdate $stime";
 
            next;
        }
    }
     
    if ( $pendingjobs ) {
        if ($line =~ /^\s+\d+\s+/) {
#           157709     0 kl43bd32_5 b3101ek      qw    06/22/2004 19:56:37
            ($dum0, $jobid, $dum1, $jobname, $user, $status, $sdate, $stime, $rest ) = split(/\s+/, $line);
 
#           print "$jobid $dum1 $jobname $user $status $rest";
            $users_jobs_queued{$user} ++;
            $queue_rank ++;
            $hoh_lrmsjobs{$jobid}{"job_state"} = "Q";
            $hoh_lrmsjobs{$jobid}{"job_sub"} = "$sdate $stime";
        }
    }
}
close SGENODESOUT;

# $hoh_lrmsjobs{$jobid}{"job_sub"} and "job_start"  is not treated yet by the
# QJU modul
#LRMSPORT>



#<LRMSPORT
# scan the LRMS and count the number of running jobs with their multiciplity
# and the number or queued jobs 
# count the jobs per unix users and the number of queueing jobs per users
# These values may be needed for the calculation of the per user freecpus
# useful variables: $users_jobs_queued{$username}, $users_jobs_running{$username}
# required values for the queue entry: $totalrunning_in_the_queue, $totalqueued

#
#Count the running and queued GRID jobs (a subset of the SGE jobs)
#running jobs are counted with their multiciplity  
#

$gridrunning = 0;
$gridqueued=  0;
foreach my $ID (keys %hoh_gmjobs){	 
	 next unless  $hoh_gmjobs{$ID}{"status"} eq "INLRMS";
	 $sgejobid=$hoh_gmjobs{$ID}{"localid"};	 	
	 if ($hoh_lrmsjobs{$sgejobid}{"job_state"} eq "R"){	      
	     $gridrunning+=$hoh_gmjobs{$ID}{"count"};
	     my $rusage;
	     &infosys_shared::read_jobinf("$sgejobinf_command $sgejobid", \$rusage);
#  	     cpu=06:01:07, mem=202.10015 GBs, io=0.00000, vmem=12.94M, maxvmem=12.94M
	     ($cpu, $mem, $io, $vmem, $mvmem ) = split(', ', $rusage);
	     @m = $cpu =~ /(\d+):*/g;
	     if ( @m == 4 ) { # 1st digit is days
		 $cpumin = $m[0]*60*24+$m[1]*60+$m[2];
	     }
	     elsif ( @m == 3 ) { # 1st digit is hours
		 $cpumin = $m[0]*60+$m[1];
	     }
	     else {
		 $cpumin = $cpu;
	     }
	     $hoh_lrmsjobs{$sgejobid}{"used.cput"} = $cpumin;
	     $vmem =~ /vmem=(\d+)\.(\d+)/;
	     $hoh_lrmsjobs{$sgejobid}{"used.mem"} = $1*1024+$2;
	 }     
# 	 GD fixme
# 	 if ($hoh_pbsjobs{$sgejobid}{"job_state"} eq "E"){	      
# 	     $gridrunning+=$hoh_gmjobs{$ID}{"count"};
# 	 }	 
	 if ($hoh_lrmsjobs{$sgejobid}{"job_state"} eq "Q"){
	     $gridqueued++;
	 }                 
}


#Can any of these be determined from SGE for a running/queueing job? 
# dummy value for used memory (take care of the unit)			     	      
$hoh_lrmsjobs{"dummy13"}{"used.mem"} = "13";	      
# dummy value (in minutes) for used walltime
$hoh_lrmsjobs{"dummy13"}{"used.walltime"} = 13;
# dummy value (in minutes) for used cputime (needs to be given in minutes)
$hoh_lrmsjobs{"dummy13"}{"used.cput"} = 13;		
# dummy value (in minutes) for requested cputime
$hoh_lrmsjobs{"dummy13"}{"req.cput"} = "66";
#nordugrid-job-executionnodes, notice the 'syntax' of multiple nodes
$hoh_lrmsjobs{"dummy13"}{"exec_host"} = "dummy1+dummy2+dummy3";
# comment string from the LRMS
$hoh_lrmsjobs{"dummy13"}{"comment"} = "Running/queuing in dummy LRMS"; 	
# dummy value for queue position
$hoh_lrmsjobs{"dummy13"}{"rank"} = 13;



#<LRMSPORT
# Count the running and queued GRID jobs in the LRMS (a subset of all LRMS jobs)
# running jobs are counted with their multiciplity
# you can loop through the hoh_gmjobs and use the "dummy13"=$hoh_gmjobs{$ID}{"localid"}
# and the $hoh_lrmsjobs{"dummy13"}{"job_state"} to find queuing and running grid jobs
# in the LRMS
# Provide the $gridrunning, $gridqueued numbers

# nordugrid-queue-gridrunning: done ($gridrunning)
# nordugrid-queue-gridqueued: done ($gridqueued)
# nordugrid-queue-running: done  ($totalrunning_in_the_queue)
# nordugrid-queue-queued: done ($totalqueued)

# GD for SGE whole cluster is the queue, took code for cluster-sge.pl
# Calculate $lrms_queued, the number of jobs (both grid and nongrid) 
# being queued in the LRMS 


#read the qstat information about the queued jobs
unless (open QSTATOUTPUT,  "$qstat_command -s p 2>/dev/null  |") {
    $loglevel and  &infosys_shared::write_log("Error in executing qstat");
    die "Error in executing qstat\n";
}
    
my @queued_sge_jobs;
while (my $line= <QSTATOUTPUT>) {       
      if ($line =~ m/^\s\d+/) {
         push @queued_sge_jobs, $line;		     	 
      	 }
      }
	    
close QSTATOUTPUT;
$totalqueued = @queued_sge_jobs;
#LRMSPORT>


if ($config{loglevel} == 2) {  
   my $runtime = time - $timestamp; 
   &infosys_shared::write_log("LRMS I/O time: $runtime");  
}



#<LRMSPORT
# nordugrid-queue-status
# determine the $lrms_queue_status from the LRMS
# set it to 'active' if the queue can accept jobs
# SGE port is NEEDED!!!
$lrms_queue_status = "active";
#LRMSPORT>


# loop over the mapped local unix users and determine the LRMS-dependent
# freecpus and queuelength for each of them. 
# Use the %users hash which contains the grid user -> local unix user
# mapping info:  $users{Grid_User_SN}=mapped_unix_id
# Take into account queue limits (cpu & time),
# physically available cpus (values determined above) or use LRMS specific method
# Store the result in two hashes, take care of the syntax of the freecpus: cpus[:minutes] 
# here the :minutes part is optional and used for specifying time limit on CPUs.
# $freecpus_for_mapped_users{unix_username}="13:33"
# $users_jobs_queued{$unix_username}=13


$systemlimit = $totalcpus - $usedcpus;
foreach $mapped_unixid ( values %users ) {
    next  if (defined $users_freecpus{$mapped_unixid});
    #nordugrid-authuser-freecpus    
    $users_freecpus{$mapped_unixid} = "$systemlimit";
    #nordugrid-authuser-queuelength: done ($users_jobs_queued{$mapped_unixid})    
}
    
#LRMSPORT>

&queueldif;


&jobsldif;


&usersldif;



my $runtime =  time - $^T;
if ($config{loglevel} == 2) {
   &infosys_shared::write_log("execution time: $runtime");
}
elsif ($config{loglevel} = 1 and $runtime >= 4 ) {  
   &infosys_shared::write_log("SLOW script: $runtime");  
}
