#!/usr/bin/perl 

package remote_infosys_shared;

# 'use lib' order matters! inserted at front of include path each time - sort
# in 'increasing importance' order (environment should overrule all other, etc)
use lib "$ENV{PWD}/";
use lib "/opt/nordugrid/libexec";
# 'Make' (*not* autoconf!) expands this one to avoid hardcoding. Please refer to:
# http://www.gnu.org/software/autoconf/manual/autoconf-2.57/html_chapter/autoconf_4.html#SEC22
use lib "/opt/nordugrid/libexec";
use lib "$ENV{NORDUGRID_LOCATION}/libexec";

# we need write_log etc from ordinary infosys_shared
use infosys_shared;
# import resource gateway configuration
use gw_shared qw(%gw_config);

sub remote_read_pbsnodes ($$){ 
   #processing the pbsnodes output by using a hash of hashes %hoh_pbsnodes (referrenced by $hashref)
   my ($pbsnodes_command,$hashref) =@_;

   if(! $gw_config{gateway_enabled}) {
	write_log("ERROR: expecting remote LRMS but gateway is not configured!");
	die "Error in executing remote pbsnodes";
   }

   #print("remote pbsnodes query: $gw_config{remote_exec} -c $gw_config{gateway_conf} '$pbsnodes_command -a'\n");
   unless (open PBSNODESOUT, "$gw_config{remote_exec} -c $gw_config{gateway_conf} '$pbsnodes_command -a' |") {
	write_log("error in executing pbsnodes remotely");
	die "Error in executing pbsnodes remotely";
   }
   

   while (my $line= <PBSNODESOUT>) {	    
   	 if ($line =~ /^$/) {next}; 
   	 if ($line =~ /^([\w\-]+)/) {		 
   	    $nodeid=$1;
   	    next;	    
   	 }
   	 if ($line =~ / = /)  {
   	     ($node_var,$node_value) = split (/ = /, $line);
   	     $node_var =~ s/\s+//g;
   	     chop $node_value;  	     
   	 }	     
   	 ${$hashref}{$nodeid}{$node_var}=$node_value;     
   } 
   close PBSNODESOUT;
}

# TODO: implement similarly as pbsnodes above
sub remote_read_sgenodes ($$){ 
   #processing the SGE qstat -f output by using a hash %h_sgenodes (referrenced by $hashref)
#GD SGE output
## queuename            qtype used/tot. load_avg arch      states
##----------------------------------------------------------------------------
##lxsrv009             B     2/2       2.40     glinux    d

   my ($sgenodes_command,$hashref) =@_;
   unless (open SGENODESOUT,  "$sgenodes_command |") {
      write_log("error in executing $sgenodes_command");
      die "Error in executing $sgenodes_command";
   }
   while (my $line= <SGENODESOUT>) {	    
   	 if ($line =~ /^$/) {next}; 
	 
   	 if ($line =~ /^lxsrv/) {
	     ($nodeid, $nodetype, $rest ) = split(/\s+/, $line);
	     if ( ! ($nodetype =~ /B/ )) {next}; # only nodes for batch

	     ${$hashref}{$nodeid} = $rest;     
	 } 
     }
   close SGENODESOUT;
}


# TODO: implement similarly as pbsnodes above
sub read_jobinf ($$){ 
# read detailed SGE jobinfo to obtain resource usage
#
# a call as: qstat -F -j 154138
# gives : usage    1:                  cpu=06:01:07, mem=202.10015 GBs, io=0.00000, vmem=12.94M, maxvmem=12.94M

   my ($jid, $usgref) = @_;
#   print "$sgejobinf_command $jid ";
   $sgejobinf_command = "qstat -F -j "; 
   unless (open SGEJOBINFOUT,  "$sgejobinf_command $jid |") {
      write_log("error in executing $sgejobinf_command");
      die "Error in executing $sgejobinf_command";
   }
   while (my $linein= <SGEJOBINFOUT>) {	    
   	 if ($linein =~ /^$/) {next}; 
	 
   	 if ($linein =~ /usage/) {
	     chomp($linein);
	     ($dum, $usg ) = split(":", $linein,2);
	     $usg =~ s/^\s*//;
	     
	     last;
#	     print "$ usg \n";
	 } 
     }
   ${$usgref} = $usg;
   close SGEJOBINFOUT;
}

   
1;
